@@ -25,4 +25,15 @@ def cli(env, volume_id, reason, immediate):
     if not (env.skip_confirmations or formatting.no_going_back(volume_id)):
         raise exceptions.CLIAbort('Aborted')
 
-    block_storage_manager.cancel_block_volume(volume_id, reason, immediate)
+    cancelled = block_storage_manager.cancel_block_volume(volume_id,
+                                                          reason, immediate)
+
+    if cancelled:
+        if immediate:
+            click.echo('Block volume with id %s has been marked'
+                       ' for immediate cancellation' % volume_id)
+        else:
+            click.echo('Block volume with id %s has been marked'
+                       ' for cancellation' % volume_id)
+    else:
+        click.echo('Unable to cancel block volume %s' % volume_id)
