@@ -2,13 +2,13 @@
 # :license: MIT, see LICENSE for more details.
 from os import path
 
+import click
+
 import SoftLayer
 from SoftLayer.CLI import environment
 from SoftLayer.CLI import formatting
 from SoftLayer.CLI import helpers
 
-import click
-
 
 @click.command()
 @click.argument('identifier')
@@ -26,10 +26,10 @@ def cli(env, identifier, out_file):
     key = mgr.get_key(key_id)
 
     if out_file:
-        with open(path.expanduser(out_file), 'w') as pub_file:
+        with open(path.expanduser(out_file), 'w', encoding="utf-8") as pub_file:
             pub_file.write(key['key'])
 
-    table = formatting.KeyValueTable(['Name', 'Value'])
+    table = formatting.KeyValueTable(['name', 'value'])
     table.add_row(['id', key['id']])
     table.add_row(['label', key.get('label')])
     table.add_row(['notes', key.get('notes', '-')])
