@@ -1,11 +1,11 @@
 """Add and upload SSL certificate details."""
 # :license: MIT, see LICENSE for more details.
 
+import click
+
 import SoftLayer
 from SoftLayer.CLI import environment
 
-import click
-
 
 @click.command()
 @click.option('--crt',
@@ -28,15 +28,19 @@ def cli(env, crt, csr, icc, key, notes):
         'certificateSigningRequest': '',
         'notes': notes,
     }
-    template['certificate'] = open(crt).read()
-    template['privateKey'] = open(key).read()
-    if csr:
-        body = open(csr).read()
-        template['certificateSigningRequest'] = body
+    with open(crt, encoding="utf-8") as file_crt:
+        template['certificate'] = file_crt.read()
+    with open(key, encoding="utf-8") as file_key:
+        template['privateKey'] = file_key.read()
+    with open(csr, encoding="utf-8") as file_csr:
+        if csr:
+            body = file_csr.read()
+            template['certificateSigningRequest'] = body
 
-    if icc:
-        body = open(icc).read()
-        template['intermediateCertificate'] = body
+    with open(icc, encoding="utf-8") as file_icc:
+        if icc:
+            body = file_icc.read()
+            template['intermediateCertificate'] = body
 
     manager = SoftLayer.SSLManager(env.client)
     manager.add_certificate(template)
