@@ -25,5 +25,22 @@ getObject = {
         }
     ],
     'hardware': [],
-    'usableIpAddressCount': 22
+    'usableIpAddressCount': 22,
+    'note': 'test note',
+    'tagReferences': [
+        {'id': 1000123,
+         'resourceTableId': 1234,
+         'tag': {'id': 100123,
+                 'name': 'subnet: test tag'},
+         }
+    ],
+    'ipAddresses': [
+        {'id': 123456,
+         'ipAddress': '16.26.26.25'},
+        {'id': 123457,
+         'ipAddress': '16.26.26.26'}]
 }
+
+editNote = True
+setTags = True
+cancel = True
