@@ -32,7 +32,7 @@ def has_firewall(vlan):
 class FirewallManager(utils.IdentifierMixin, object):
     """Manages SoftLayer firewalls
 
-    See product information here: http://www.softlayer.com/firewalls
+    See product information here: https://www.ibm.com/cloud/network-security
 
     :param SoftLayer.API.BaseClient client: the client instance
 
@@ -282,10 +282,23 @@ class FirewallManager(utils.IdentifierMixin, object):
         """Edit the rules for standard firewall.
 
         :param integer firewall_id: the instance ID of the standard firewall
-        :param dict rules: the rules to be pushed on the firewall
+        :param list rules: the rules to be pushed on the firewall as defined by
+                           SoftLayer_Network_Firewall_Update_Request_Rule
         """
 
         rule_svc = self.client['Network_Firewall_Update_Request']
         template = {'networkComponentFirewallId': firewall_id, 'rules': rules}
 
         return rule_svc.createObject(template)
+
+    def get_instance(self, firewall_id, mask=None):
+        """Get the firewall information
+
+        :param integer firewall_id: the instance ID of the standard firewall
+        """
+        if not mask:
+            mask = 'mask[datacenter,networkVlan]'
+
+        svc = self.client['Network_Vlan_Firewall']
+
+        return svc.getObject(id=firewall_id, mask=mask)
