@@ -4,11 +4,13 @@
 
     :license: MIT, see LICENSE for more details.
 """
-from SoftLayer import exceptions
+from SoftLayer.CLI import exceptions
+from SoftLayer.CLI import formatting
+from SoftLayer import SoftLayerAPIError
 from SoftLayer import testing
 
 import json
-import mock
+from unittest import mock as mock
 
 
 class BlockTests(testing.TestCase):
@@ -48,7 +50,7 @@ class BlockTests(testing.TestCase):
     def test_volume_set_lun_id_not_in_range(self):
         value = '-1'
         lun_mock = self.set_mock('SoftLayer_Network_Storage', 'createOrUpdateLunId')
-        lun_mock.side_effect = exceptions.SoftLayerAPIError(
+        lun_mock.side_effect = SoftLayerAPIError(
             'SoftLayer_Exception_Network_Storage_Iscsi_InvalidLunId',
             'The LUN ID specified is out of the valid range: %s [min: 0 max: 4095]' % (value))
         result = self.run_command('block volume-set-lun-id 1234 42'.split())
@@ -59,9 +61,12 @@ class BlockTests(testing.TestCase):
         result = self.run_command(['block', 'volume-detail', '1234'])
 
         self.assert_no_fail(result)
+        isinstance(json.loads(result.output)['IOPs'], float)
+        self.assert_called_with('SoftLayer_Network_Storage', 'getObject', identifier=1234)
         self.assertEqual({
             'Username': 'username',
             'LUN Id': '2',
+            'Notes': "{'status': 'available'}",
             'Endurance Tier': 'READHEAVY_TIER',
             'IOPs': 1000,
             'Snapshot Capacity (GB)': '10',
@@ -97,6 +102,26 @@ class BlockTests(testing.TestCase):
             ]
         }, json.loads(result.output))
 
+    def test_volume_detail_name_identifier(self):
+        result = self.run_command(['block', 'volume-detail', 'SL-12345'])
+        expected_filter = {
+            'iscsiNetworkStorage': {
+                'serviceResource': {
+                    'type': {
+                        'type': {'operation': '!~ ISCSI'}
+                    }
+                },
+                'storageType': {
+                    'keyName': {'operation': '*= BLOCK_STORAGE'}
+                },
+                'username': {'operation': '_= SL-12345'}
+            }
+        }
+
+        self.assert_called_with('SoftLayer_Account', 'getIscsiNetworkStorage', filter=expected_filter)
+        self.assert_called_with('SoftLayer_Network_Storage', 'getObject', identifier=100)
+        self.assert_no_fail(result)
+
     def test_volume_list(self):
         result = self.run_command(['block', 'volume-list'])
 
@@ -107,8 +132,10 @@ class BlockTests(testing.TestCase):
                 'capacity_gb': 20,
                 'datacenter': 'dal05',
                 'id': 100,
+                'IOPs': None,
                 'ip_addr': '10.1.2.3',
                 'lunId': None,
+                'notes': "{'status': 'available'}",
                 'rep_partner_count': None,
                 'storage_type': 'ENDURANCE',
                 'username': 'username',
@@ -116,6 +143,44 @@ class BlockTests(testing.TestCase):
             }],
             json.loads(result.output))
 
+    @mock.patch('SoftLayer.BlockStorageManager.list_block_volumes')
+    def test_volume_list_notes_format_output_json(self, list_mock):
+        note_mock = 'test ' * 5
+        list_mock.return_value = [
+            {'notes': note_mock}
+        ]
+
+        result = self.run_command(['--format', 'json', 'block', 'volume-list', '--columns', 'notes'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(
+            [{
+                'notes': note_mock,
+            }],
+            json.loads(result.output))
+
+    @mock.patch('SoftLayer.BlockStorageManager.list_block_volumes')
+    def test_volume_list_reduced_notes_format_output_table(self, list_mock):
+        note_mock = 'test ' * 10
+        expected_reduced_note = 'test ' * 4
+        list_mock.return_value = [
+            {'notes': note_mock}
+        ]
+        expected_table = formatting.Table(['notes'])
+        expected_table.add_row([expected_reduced_note])
+        expected_output = formatting.format_output(expected_table)+'\n'
+        result = self.run_command(['--format', 'table', 'block', 'volume-list', '--columns', 'notes'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(expected_output, result.output)
+
+    def test_volume_list_order(self):
+        result = self.run_command(['block', 'volume-list', '--order=1234567'])
+
+        self.assert_no_fail(result)
+        json_result = json.loads(result.output)
+        self.assertEqual(json_result[0]['id'], 100)
+
     @mock.patch('SoftLayer.BlockStorageManager.list_block_volumes')
     def test_volume_count(self, list_mock):
         list_mock.return_value = [
@@ -141,14 +206,6 @@ class BlockTests(testing.TestCase):
 
         self.assertEqual(2, result.exit_code)
 
-    def test_volume_order_performance_iops_not_multiple_of_100(self):
-        result = self.run_command(['block', 'volume-order',
-                                   '--storage-type=performance', '--size=20',
-                                   '--iops=122', '--os-type=linux',
-                                   '--location=dal05'])
-
-        self.assertEqual(2, result.exit_code)
-
     def test_volume_order_performance_snapshot_error(self):
         result = self.run_command(['block', 'volume-order',
                                    '--storage-type=performance', '--size=20',
@@ -182,7 +239,9 @@ class BlockTests(testing.TestCase):
                          'Order #478 placed successfully!\n'
                          ' > Performance Storage\n > Block Storage\n'
                          ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
-                         ' > 10 GB Storage Space (Snapshot Space)\n')
+                         ' > 10 GB Storage Space (Snapshot Space)\n'
+                         '\nYou may run "slcli block volume-list --order 478" to find this block volume '
+                         'after it is ready.\n')
 
     def test_volume_order_endurance_tier_not_given(self):
         result = self.run_command(['block', 'volume-order',
@@ -202,7 +261,7 @@ class BlockTests(testing.TestCase):
                     {'description': '0.25 IOPS per GB'},
                     {'description': '20 GB Storage Space'},
                     {'description': '10 GB Storage Space (Snapshot Space)'}]
-                }
+            }
         }
 
         result = self.run_command(['block', 'volume-order',
@@ -215,7 +274,9 @@ class BlockTests(testing.TestCase):
                          'Order #478 placed successfully!\n'
                          ' > Endurance Storage\n > Block Storage\n'
                          ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
-                         ' > 10 GB Storage Space (Snapshot Space)\n')
+                         ' > 10 GB Storage Space (Snapshot Space)\n'
+                         '\nYou may run "slcli block volume-list --order 478" to find this block volume '
+                         'after it is ready.\n')
 
     @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
     def test_volume_order_order_not_placed(self, order_mock):
@@ -250,7 +311,7 @@ class BlockTests(testing.TestCase):
                     {'description': 'Block Storage'},
                     {'description': '20 GB Storage Space'},
                     {'description': '200 IOPS'}]
-                }
+            }
         }
 
         result = self.run_command(['block', 'volume-order',
@@ -264,7 +325,9 @@ class BlockTests(testing.TestCase):
                          ' > Storage as a Service\n'
                          ' > Block Storage\n'
                          ' > 20 GB Storage Space\n'
-                         ' > 200 IOPS\n')
+                         ' > 200 IOPS\n'
+                         '\nYou may run "slcli block volume-list --order 10983647" to find this block volume '
+                         'after it is ready.\n')
 
     @mock.patch('SoftLayer.BlockStorageManager.order_block_volume')
     def test_volume_order_performance_manager_error(self, order_mock):
@@ -416,7 +479,7 @@ class BlockTests(testing.TestCase):
                 'items': [{'description':
                            '10 GB Storage Space (Snapshot Space)'}],
                 'status': 'PENDING_APPROVAL',
-                }
+            }
         }
 
         result = self.run_command(['block', 'snapshot-order', '1234',
@@ -444,13 +507,40 @@ class BlockTests(testing.TestCase):
 
         self.assert_no_fail(result)
 
-    def test_replicant_failover(self):
-        result = self.run_command(['block', 'replica-failover', '12345678',
-                                   '--replicant-id=5678', '--immediate'])
+    def test_assign_subnets_to_acl(self):
+        result = self.run_command(['block', 'subnets-assign', '12345',
+                                   '--subnet-id=12345678'])
 
         self.assert_no_fail(result)
-        self.assertEqual('Failover to replicant is now in progress.\n',
-                         result.output)
+
+    def test_remove_subnets_from_acl(self):
+        result = self.run_command(['block', 'subnets-remove', '12345',
+                                   '--subnet-id=12345678'])
+
+        self.assert_no_fail(result)
+
+    def test_get_subnets_in_acl(self):
+        result = self.run_command(['block', 'subnets-list', '12345'])
+
+        self.assert_no_fail(result)
+
+    def test_replicant_failover(self):
+        result = self.run_command(['block', 'replica-failover', '12345678',
+                                   '--replicant-id=5678'])
+
+        self.assert_no_fail(result)
+        self.assertEqual('Failover to replicant is now in progress.\n', result.output)
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    @mock.patch('SoftLayer.BlockStorageManager.disaster_recovery_failover_to_replicant')
+    def test_disaster_recovery_failover(self, disaster_recovery_failover_mock, confirm_mock):
+        confirm_mock.return_value = True
+        disaster_recovery_failover_mock.return_value = True
+        result = self.run_command(['block', 'disaster-recovery-failover', '12345678',
+                                   '--replicant-id=5678'])
+
+        self.assert_no_fail(result)
+        self.assertIn('Disaster Recovery Failover to replicant is now in progress.\n', result.output)
 
     def test_replication_locations(self):
         result = self.run_command(['block', 'replica-locations', '1234'])
@@ -511,10 +601,19 @@ class BlockTests(testing.TestCase):
         self.assertEqual('Failover operation could not be initiated.\n',
                          result.output)
 
-    def test_replicant_failback(self):
-        result = self.run_command(['block', 'replica-failback', '12345678',
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_disaster_recovery_failover_aborted(self, confirm_mock):
+        confirm_mock.return_value = False
+
+        result = self.run_command(['block', 'disaster-recovery-failover', '12345678',
                                    '--replicant-id=5678'])
 
+        self.assertEqual(result.exit_code, 2)
+        self.assertIsInstance(result.exception, exceptions.CLIAbort)
+
+    def test_replicant_failback(self):
+        result = self.run_command(['block', 'replica-failback', '12345678'])
+
         self.assert_no_fail(result)
         self.assertEqual('Failback from replicant is now in progress.\n',
                          result.output)
@@ -523,8 +622,7 @@ class BlockTests(testing.TestCase):
     def test_replicant_failback_unsuccessful(self, failback_mock):
         failback_mock.return_value = False
 
-        result = self.run_command(['block', 'replica-failback', '12345678',
-                                   '--replicant-id=5678'])
+        result = self.run_command(['block', 'replica-failback', '12345678'])
 
         self.assertEqual('Failback operation could not be initiated.\n',
                          result.output)
@@ -633,7 +731,8 @@ class BlockTests(testing.TestCase):
                                       duplicate_size=250, duplicate_iops=None,
                                       duplicate_tier_level=2,
                                       duplicate_snapshot_size=20,
-                                      hourly_billing_flag=True)
+                                      hourly_billing_flag=True,
+                                      dependent_duplicate=False)
         self.assert_no_fail(result)
         self.assertEqual(result.output,
                          'Order #24602 placed successfully!\n'
@@ -673,3 +772,53 @@ class BlockTests(testing.TestCase):
     def test_set_password(self):
         result = self.run_command(['block', 'access-password', '1234', '--password=AAAAA'])
         self.assert_no_fail(result)
+
+    @mock.patch('SoftLayer.BlockStorageManager.list_block_volume_limit')
+    def test_volume_limit(self, list_mock):
+        list_mock.return_value = [
+            {
+                "datacenterName": "global",
+                "maximumAvailableCount": 300,
+                "provisionedCount": 100
+            }]
+
+        result = self.run_command(['block', 'volume-limits'])
+        self.assert_no_fail(result)
+
+    def test_dupe_refresh(self):
+        result = self.run_command(['block', 'volume-refresh', '102', '103'])
+
+        self.assert_no_fail(result)
+
+    def test_dep_dupe_convert(self):
+        result = self.run_command(['block', 'volume-convert', '102'])
+
+        self.assert_no_fail(result)
+
+    @mock.patch('SoftLayer.BlockStorageManager.volume_set_note')
+    def test_volume_set_note(self, set_note):
+        set_note.return_value = True
+
+        result = self.run_command(['block', 'volume-set-note', '102', '--note=testing'])
+
+        self.assert_no_fail(result)
+        self.assertIn("successfully!", result.output)
+
+    @mock.patch('SoftLayer.BlockStorageManager.volume_set_note')
+    def test_volume_not_set_note(self, set_note):
+        set_note.return_value = False
+
+        result = self.run_command(['block', 'volume-set-note', '102', '--note=testing'])
+
+        self.assert_no_fail(result)
+        self.assertIn("Note could not be set!", result.output)
+
+    @mock.patch('SoftLayer.BlockStorageManager.get_volume_snapshot_notification_status')
+    def test_snapshot_get_notification_status(self, status):
+        status.side_effect = [None, 1, 0]
+        expected = ['Enabled', 'Enabled', 'Disabled']
+
+        for expect in expected:
+            result = self.run_command(['block', 'snapshot-get-notification-status', '999'])
+            self.assert_no_fail(result)
+            self.assertIn(expect, result.output)
