@@ -4,10 +4,13 @@
 
     :license: MIT, see LICENSE for more details.
 """
+from SoftLayer.CLI import exceptions
+from SoftLayer.CLI import formatting
+from SoftLayer import SoftLayerError
 from SoftLayer import testing
 
 import json
-import mock
+from unittest import mock as mock
 
 
 class FileTests(testing.TestCase):
@@ -49,10 +52,49 @@ class FileTests(testing.TestCase):
                 'username': 'user',
                 'active_transactions': None,
                 'mount_addr': '127.0.0.1:/TEST',
+                'notes': None,
                 'rep_partner_count': None
             }],
             json.loads(result.output))
 
+    def test_volume_list_order(self):
+        result = self.run_command(['file', 'volume-list', '--order=1234567'])
+
+        self.assert_no_fail(result)
+        json_result = json.loads(result.output)
+        self.assertEqual(json_result[0]['id'], 1)
+
+    @mock.patch('SoftLayer.FileStorageManager.list_file_volumes')
+    def test_volume_list_notes_format_output_json(self, list_mock):
+        note_mock = 'test ' * 5
+        list_mock.return_value = [
+            {'notes': note_mock}
+        ]
+
+        result = self.run_command(['--format', 'json', 'file', 'volume-list', '--columns', 'notes'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(
+            [{
+                'notes': note_mock,
+            }],
+            json.loads(result.output))
+
+    @mock.patch('SoftLayer.FileStorageManager.list_file_volumes')
+    def test_volume_list_reduced_notes_format_output_table(self, list_mock):
+        note_mock = 'test ' * 10
+        expected_reduced_note = 'test ' * 4
+        list_mock.return_value = [
+            {'notes': note_mock}
+        ]
+        expected_table = formatting.Table(['notes'])
+        expected_table.add_row([expected_reduced_note])
+        expected_output = formatting.format_output(expected_table) + '\n'
+        result = self.run_command(['--format', 'table', 'file', 'volume-list', '--columns', 'notes'])
+
+        self.assert_no_fail(result)
+        self.assertEqual(expected_output, result.output)
+
     @mock.patch('SoftLayer.FileStorageManager.list_file_volumes')
     def test_volume_count(self, list_mock):
         list_mock.return_value = [
@@ -94,6 +136,31 @@ class FileTests(testing.TestCase):
         self.assert_called_with('SoftLayer_Billing_Item', 'cancelItem',
                                 args=(False, True, None))
 
+    def test_volume_cancel_with_billing_item(self):
+        result = self.run_command([
+            '--really', 'file', 'volume-cancel', '1234'])
+
+        self.assert_no_fail(result)
+        self.assertEqual('File volume with id 1234 has been marked'
+                         ' for cancellation\n', result.output)
+        self.assert_called_with('SoftLayer_Network_Storage', 'getObject')
+
+    def test_volume_cancel_without_billing_item(self):
+        p_mock = self.set_mock('SoftLayer_Network_Storage', 'getObject')
+        p_mock.return_value = {
+            "accountId": 1234,
+            "capacityGb": 20,
+            "createDate": "2015-04-29T06:55:55-07:00",
+            "id": 11111,
+            "nasType": "NAS",
+            "username": "SL01SEV307608_1"
+        }
+
+        result = self.run_command([
+            '--really', 'file', 'volume-cancel', '1234'])
+
+        self.assertIsInstance(result.exception, SoftLayerError)
+
     def test_volume_detail(self):
         result = self.run_command(['file', 'volume-detail', '1234'])
 
@@ -111,6 +178,7 @@ class FileTests(testing.TestCase):
             'Data Center': 'dal05',
             'Type': 'ENDURANCE',
             'ID': 100,
+            'Notes': "{'status': 'available'}",
             '# of Active Transactions': '1',
             'Ongoing Transaction': 'This is a buffer time in which the customer may cancel the server',
             'Replicant Count': '1',
@@ -137,6 +205,24 @@ class FileTests(testing.TestCase):
             ]
         }, json.loads(result.output))
 
+    def test_volume_detail_name_identifier(self):
+        result = self.run_command(['file', 'volume-detail', 'SL-12345'])
+        expected_filter = {
+            'nasNetworkStorage': {
+                'serviceResource': {
+                    'type': {
+                        'type': {'operation': '!~ NAS'}
+                    }
+                },
+                'storageType': {
+                    'keyName': {'operation': '*= FILE_STORAGE'}
+                },
+                'username': {'operation': '_= SL-12345'}}}
+
+        self.assert_called_with('SoftLayer_Account', 'getNasNetworkStorage', filter=expected_filter)
+        self.assert_called_with('SoftLayer_Network_Storage', 'getObject', identifier=1)
+        self.assert_no_fail(result)
+
     def test_volume_order_performance_iops_not_given(self):
         result = self.run_command(['file', 'volume-order',
                                    '--storage-type=performance', '--size=20',
@@ -177,7 +263,9 @@ class FileTests(testing.TestCase):
                          'Order #478 placed successfully!\n'
                          ' > Performance Storage\n > File Storage\n'
                          ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
-                         ' > 10 GB Storage Space (Snapshot Space)\n')
+                         ' > 10 GB Storage Space (Snapshot Space)\n'
+                         '\nYou may run "slcli file volume-list --order 478" to find this file volume after it is '
+                         'ready.\n')
 
     def test_volume_order_endurance_tier_not_given(self):
         result = self.run_command(['file', 'volume-order',
@@ -210,7 +298,9 @@ class FileTests(testing.TestCase):
                          'Order #478 placed successfully!\n'
                          ' > Endurance Storage\n > File Storage\n'
                          ' > 0.25 IOPS per GB\n > 20 GB Storage Space\n'
-                         ' > 10 GB Storage Space (Snapshot Space)\n')
+                         ' > 10 GB Storage Space (Snapshot Space)\n'
+                         '\nYou may run "slcli file volume-list --order 478" to find this file volume after it is '
+                         'ready.\n')
 
     @mock.patch('SoftLayer.FileStorageManager.order_file_volume')
     def test_volume_order_order_not_placed(self, order_mock):
@@ -261,7 +351,9 @@ class FileTests(testing.TestCase):
                          ' > File Storage\n'
                          ' > 20 GB Storage Space\n'
                          ' > 0.25 IOPS per GB\n'
-                         ' > 10 GB Storage Space (Snapshot Space)\n')
+                         ' > 10 GB Storage Space (Snapshot Space)\n'
+                         '\nYou may run "slcli file volume-list --order 479" to find this file volume after it is '
+                         'ready.\n')
 
     @mock.patch('SoftLayer.FileStorageManager.order_file_volume')
     def test_volume_order_performance_manager_error(self, order_mock):
@@ -418,7 +510,7 @@ class FileTests(testing.TestCase):
 
     def test_replicant_failover(self):
         result = self.run_command(['file', 'replica-failover', '12345678',
-                                   '--replicant-id=5678', '--immediate'])
+                                   '--replicant-id=5678'])
 
         self.assert_no_fail(result)
         self.assertEqual('Failover to replicant is now in progress.\n',
@@ -434,10 +526,29 @@ class FileTests(testing.TestCase):
         self.assertEqual('Failover operation could not be initiated.\n',
                          result.output)
 
-    def test_replicant_failback(self):
-        result = self.run_command(['file', 'replica-failback', '12345678',
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    @mock.patch('SoftLayer.FileStorageManager.disaster_recovery_failover_to_replicant')
+    def test_disaster_recovery_failover(self, disaster_recovery_failover_mock, confirm_mock):
+        confirm_mock.return_value = True
+        disaster_recovery_failover_mock.return_value = True
+        result = self.run_command(['file', 'disaster-recovery-failover', '12345678', '--replicant-id=5678'])
+
+        self.assert_no_fail(result)
+        self.assertIn('Disaster Recovery Failover to replicant is now in progress.\n', result.output)
+
+    @mock.patch('SoftLayer.CLI.formatting.confirm')
+    def test_disaster_recovery_failover_aborted(self, confirm_mock):
+        confirm_mock.return_value = False
+
+        result = self.run_command(['file', 'disaster-recovery-failover', '12345678',
                                    '--replicant-id=5678'])
 
+        self.assertEqual(result.exit_code, 2)
+        self.assertIsInstance(result.exception, exceptions.CLIAbort)
+
+    def test_replicant_failback(self):
+        result = self.run_command(['file', 'replica-failback', '12345678'])
+
         self.assert_no_fail(result)
         self.assertEqual('Failback from replicant is now in progress.\n',
                          result.output)
@@ -446,8 +557,7 @@ class FileTests(testing.TestCase):
     def test_replicant_failback_unsuccessful(self, failback_mock):
         failback_mock.return_value = False
 
-        result = self.run_command(['file', 'replica-failback', '12345678',
-                                   '--replicant-id=5678'])
+        result = self.run_command(['file', 'replica-failback', '12345678'])
 
         self.assertEqual('Failback operation could not be initiated.\n',
                          result.output)
@@ -605,7 +715,8 @@ class FileTests(testing.TestCase):
                                       duplicate_size=250, duplicate_iops=None,
                                       duplicate_tier_level=2,
                                       duplicate_snapshot_size=20,
-                                      hourly_billing_flag=True)
+                                      hourly_billing_flag=True,
+                                      dependent_duplicate=False)
         self.assert_no_fail(result)
         self.assertEqual(result.output,
                          'Order #24602 placed successfully!\n'
@@ -641,3 +752,52 @@ class FileTests(testing.TestCase):
         self.assert_no_fail(result)
         self.assertEqual('Order #24602 placed successfully!\n > Storage as a Service\n > 1000 GBs\n > 4 IOPS per GB\n',
                          result.output)
+
+    @mock.patch('SoftLayer.FileStorageManager.list_file_volume_limit')
+    def test_volume_limit(self, list_mock):
+        list_mock.return_value = [
+            {
+                'datacenterName': 'global',
+                'maximumAvailableCount': 300,
+                'provisionedCount': 100
+            }]
+        result = self.run_command(['file', 'volume-limits'])
+        self.assert_no_fail(result)
+
+    def test_dupe_refresh(self):
+        result = self.run_command(['file', 'volume-refresh', '102', '103'])
+
+        self.assert_no_fail(result)
+
+    def test_dep_dupe_convert(self):
+        result = self.run_command(['file', 'volume-convert', '102'])
+
+        self.assert_no_fail(result)
+
+    @mock.patch('SoftLayer.FileStorageManager.volume_set_note')
+    def test_volume_set_note(self, set_note):
+        set_note.return_value = True
+
+        result = self.run_command(['file', 'volume-set-note', '102', '--note=testing'])
+
+        self.assert_no_fail(result)
+        self.assertIn("successfully!", result.output)
+
+    @mock.patch('SoftLayer.FileStorageManager.volume_set_note')
+    def test_volume_not_set_note(self, set_note):
+        set_note.return_value = False
+
+        result = self.run_command(['file', 'volume-set-note', '102', '--note=testing'])
+
+        self.assert_no_fail(result)
+        self.assertIn("Note could not be set!", result.output)
+
+    @mock.patch('SoftLayer.FileStorageManager.get_volume_snapshot_notification_status')
+    def test_snapshot_get_notification_status(self, status):
+        status.side_effect = [None, 1, 0]
+        expected = ['Enabled', 'Enabled', 'Disabled']
+
+        for expect in expected:
+            result = self.run_command(['file', 'snapshot-get-notification-status', '999'])
+            self.assert_no_fail(result)
+            self.assertIn(expect, result.output)
