@@ -1,11 +1,10 @@
 """
     SoftLayer.tests.CLI.modules.ticket_tests
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
     :license: MIT, see LICENSE for more details.
 """
 import json
-import mock
+from unittest import mock as mock
 
 from SoftLayer.CLI import exceptions
 from SoftLayer.CLI import formatting
@@ -14,6 +13,23 @@ from SoftLayer.managers import TicketManager
 from SoftLayer import testing
 
 
+class FakeTTY():
+    """A fake object to fake STD input"""
+
+    def __init__(self, isatty=False, read="Default Output"):
+        """Sets isatty and read"""
+        self._isatty = isatty
+        self._read = read
+
+    def isatty(self):
+        """returns self.isatty"""
+        return self._isatty
+
+    def read(self):
+        """returns self.read"""
+        return self._read
+
+
 class TicketTests(testing.TestCase):
 
     def test_list(self):
@@ -21,6 +37,7 @@ class TicketTests(testing.TestCase):
 
         expected = [{
             'assigned_user': 'John Smith',
+            'Case_Number': 'CS123456',
             'id': 102,
             'last_edited': '2013-08-01T14:16:47-07:00',
             'priority': 0,
@@ -34,6 +51,7 @@ class TicketTests(testing.TestCase):
         result = self.run_command(['ticket', 'detail', '1'])
 
         expected = {
+            'Case_Number': 'CS123456',
             'created': '2013-08-01T14:14:04-07:00',
             'edited': '2013-08-01T14:16:47-07:00',
             'id': 100,
@@ -41,8 +59,8 @@ class TicketTests(testing.TestCase):
             'status': 'Closed',
             'title': 'Cloud Instance Cancellation - 08/01/13',
             'update 1': 'a bot says something',
-            'update 2': 'By John Smith\nuser says something',
-            'update 3': 'By emp1 (Employee)\nemployee says something',
+            'update 2': 'By John Smith user says something',
+            'update 3': 'By emp1 (Employee) employee says something',
         }
         self.assert_no_fail(result)
         self.assertEqual(json.loads(result.output), expected)
@@ -55,7 +73,6 @@ class TicketTests(testing.TestCase):
         self.assert_no_fail(result)
 
         args = ({'subjectId': 1000,
-                 'contents': 'ticket body',
                  'assignedUserId': 12345,
                  'title': 'Test'}, 'ticket body')
 
@@ -70,7 +87,6 @@ class TicketTests(testing.TestCase):
         self.assert_no_fail(result)
 
         args = ({'subjectId': 1000,
-                 'contents': 'ticket body',
                  'assignedUserId': 12345,
                  'title': 'Test',
                  'priority': 1}, 'ticket body')
@@ -87,7 +103,6 @@ class TicketTests(testing.TestCase):
         self.assert_no_fail(result)
 
         args = ({'subjectId': 1000,
-                 'contents': 'ticket body',
                  'assignedUserId': 12345,
                  'title': 'Test'}, 'ticket body')
 
@@ -101,19 +116,33 @@ class TicketTests(testing.TestCase):
                                 identifier=100)
 
     @mock.patch('click.edit')
-    def test_create_no_body(self, edit_mock):
+    @mock.patch('click.get_text_stream')
+    def test_create_no_body(self, isatty_mock, edit_mock):
+        fake_tty = FakeTTY(True, "TEST")
+        isatty_mock.return_value = fake_tty
         edit_mock.return_value = 'ticket body'
-        result = self.run_command(['ticket', 'create', '--title=Test',
-                                   '--subject-id=1000'])
+        result = self.run_command(['ticket', 'create', '--title=Test', '--subject-id=1000'])
         self.assert_no_fail(result)
 
         args = ({'subjectId': 1000,
-                 'contents': 'ticket body',
                  'assignedUserId': 12345,
                  'title': 'Test'}, 'ticket body')
 
-        self.assert_called_with('SoftLayer_Ticket', 'createStandardTicket',
-                                args=args)
+        self.assert_called_with('SoftLayer_Ticket', 'createStandardTicket', args=args)
+
+    @mock.patch('click.get_text_stream')
+    def test_create_no_body_stdin(self, isatty_mock):
+        fake_tty = FakeTTY(False, "TEST TICKET BODY")
+        isatty_mock.return_value = fake_tty
+        result = self.run_command(['ticket', 'create', '--title=Test', '--subject-id=1000'])
+        print(result.output)
+        self.assert_no_fail(result)
+
+        args = ({'subjectId': 1000,
+                 'assignedUserId': 12345,
+                 'title': 'Test'}, 'TEST TICKET BODY')
+
+        self.assert_called_with('SoftLayer_Ticket', 'createStandardTicket', args=args)
 
     def test_subjects(self):
         list_expected_ids = [1001, 1002, 1003, 1004, 1005]
@@ -239,6 +268,7 @@ class TicketTests(testing.TestCase):
     def test_init_ticket_results_asigned_user(self):
         mock = self.set_mock('SoftLayer_Ticket', 'getObject')
         mock.return_value = {
+            "serviceProviderResourceId": "CS12345",
             "id": 100,
             "title": "Simple Title",
             "priority": 1,
@@ -296,8 +326,35 @@ class TicketTests(testing.TestCase):
         self.assert_called_with('SoftLayer_Ticket', 'addUpdate', args=({'entry': 'Testing'},), identifier=100)
 
     @mock.patch('click.edit')
-    def test_ticket_update_no_body(self, edit_mock):
+    @mock.patch('click.get_text_stream')
+    def test_ticket_update_no_body(self, isatty_mock, edit_mock):
+        fake_tty = FakeTTY(True, "TEST TICKET BODY")
+        isatty_mock.return_value = fake_tty
         edit_mock.return_value = 'Testing1'
         result = self.run_command(['ticket', 'update', '100'])
         self.assert_no_fail(result)
         self.assert_called_with('SoftLayer_Ticket', 'addUpdate', args=({'entry': 'Testing1'},), identifier=100)
+
+    @mock.patch('click.get_text_stream')
+    def test_ticket_update_no_body_stdin(self, isatty_mock):
+        fake_tty = FakeTTY(False, "TEST TICKET BODY")
+        isatty_mock.return_value = fake_tty
+        result = self.run_command(['ticket', 'update', '100'])
+        self.assert_no_fail(result)
+        self.assert_called_with('SoftLayer_Ticket', 'addUpdate',
+                                args=({'entry': 'TEST TICKET BODY'},), identifier=100)
+
+    def test_ticket_json(self):
+        result = self.run_command(['--format=json', 'ticket', 'detail', '1'])
+        expected = {'Case_Number': 'CS123456',
+                    'created': '2013-08-01T14:14:04-07:00',
+                    'edited': '2013-08-01T14:16:47-07:00',
+                    'id': 100,
+                    'priority': 'No Priority',
+                    'status': 'Closed',
+                    'title': 'Cloud Instance Cancellation - 08/01/13',
+                    'update 1': 'a bot says something',
+                    'update 2': 'By John Smith user says something',
+                    'update 3': 'By emp1 (Employee) employee says something'}
+        self.assert_no_fail(result)
+        self.assertEqual(json.loads(result.output), expected)
