@@ -5,12 +5,12 @@
     :license: MIT, see LICENSE for more details.
 """
 import io
+import json
+from unittest import mock as mock
 import warnings
 
-import mock
 import pytest
 import requests
-import six
 
 import SoftLayer
 from SoftLayer import consts
@@ -20,7 +20,7 @@ from SoftLayer import transports
 
 def get_xmlrpc_response():
     response = requests.Response()
-    list_body = six.b('''<?xml version="1.0" encoding="utf-8"?>
+    list_body = b'''<?xml version="1.0" encoding="utf-8"?>
 <params>
 <param>
 <value>
@@ -29,7 +29,7 @@ def get_xmlrpc_response():
 </array>
 </value>
 </param>
-</params>''')
+</params>'''
     response.raw = io.BytesIO(list_body)
     response.headers['SoftLayer-Total-Items'] = 10
     response.status_code = 200
@@ -40,7 +40,7 @@ class TestXmlRpcAPICall(testing.TestCase):
 
     def set_up(self):
         self.transport = transports.XmlRpcTransport(
-            endpoint_url='http://something.com',
+            endpoint_url='http://something9999999999999999999999.com',
         )
         self.response = get_xmlrpc_response()
 
@@ -48,7 +48,7 @@ class TestXmlRpcAPICall(testing.TestCase):
     def test_call(self, request):
         request.return_value = self.response
 
-        data = '''<?xml version='1.0'?>
+        data = '''<?xml version='1.0' encoding='iso-8859-1'?>
 <methodCall>
 <methodName>getObject</methodName>
 <params>
@@ -63,7 +63,7 @@ class TestXmlRpcAPICall(testing.TestCase):
 </param>
 </params>
 </methodCall>
-'''
+'''.encode()
 
         req = transports.Request()
         req.service = 'SoftLayer_Service'
@@ -71,14 +71,15 @@ class TestXmlRpcAPICall(testing.TestCase):
         resp = self.transport(req)
 
         request.assert_called_with('POST',
-                                   'http://something.com/SoftLayer_Service',
+                                   'http://something9999999999999999999999.com/SoftLayer_Service',
                                    headers={'Content-Type': 'application/xml',
                                             'User-Agent': consts.USER_AGENT},
                                    proxies=None,
                                    data=data,
                                    timeout=None,
                                    cert=None,
-                                   verify=True)
+                                   verify=True,
+                                   auth=None)
         self.assertEqual(resp, [])
         self.assertIsInstance(resp, transports.SoftLayerListResult)
         self.assertEqual(resp.total_count, 10)
@@ -114,14 +115,15 @@ class TestXmlRpcAPICall(testing.TestCase):
             headers=mock.ANY,
             timeout=None,
             cert=None,
-            verify=True)
+            verify=True,
+            auth=None)
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_identifier(self, request):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.identifier = 1234
@@ -132,14 +134,14 @@ class TestXmlRpcAPICall(testing.TestCase):
             """<member>
 <name>id</name>
 <value><int>1234</int></value>
-</member>""", kwargs['data'])
+</member>""".encode(), kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_filter(self, request):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.filter = {'TYPE': {'attribute': {'operation': '^= prefix'}}}
@@ -150,14 +152,14 @@ class TestXmlRpcAPICall(testing.TestCase):
             """<member>
 <name>operation</name>
 <value><string>^= prefix</string></value>
-</member>""", kwargs['data'])
+</member>""".encode(), kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_limit_offset(self, request):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.limit = 10
@@ -167,17 +169,17 @@ class TestXmlRpcAPICall(testing.TestCase):
         self.assertIn("""<member>
 <name>resultLimit</name>
 <value><struct>
-<member>""", kwargs['data'])
+<member>""".encode(), kwargs['data'])
         self.assertIn("""<name>limit</name>
 <value><int>10</int></value>
-</member>""", kwargs['data'])
+</member>""".encode(), kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_old_mask(self, request):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.mask = {"something": "nested"}
@@ -192,14 +194,14 @@ class TestXmlRpcAPICall(testing.TestCase):
 <value><string>nested</string></value>
 </member>
 </struct></value>
-</member>""", kwargs['data'])
+</member>""".encode(), kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_mask_call_no_mask_prefix(self, request):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.mask = "something.nested"
@@ -207,7 +209,7 @@ class TestXmlRpcAPICall(testing.TestCase):
 
         args, kwargs = request.call_args
         self.assertIn(
-            "<value><string>mask[something.nested]</string></value>",
+            "<value><string>mask[something.nested]</string></value>".encode(),
             kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
@@ -215,7 +217,7 @@ class TestXmlRpcAPICall(testing.TestCase):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.mask = "mask[something[nested]]"
@@ -223,7 +225,23 @@ class TestXmlRpcAPICall(testing.TestCase):
 
         args, kwargs = request.call_args
         self.assertIn(
-            "<value><string>mask[something[nested]]</string></value>",
+            "<value><string>mask[something[nested]]</string></value>".encode(),
+            kwargs['data'])
+
+    @mock.patch('SoftLayer.transports.requests.Session.request')
+    def test_mask_call_filteredMask(self, request):
+        request.return_value = self.response
+
+        req = transports.Request()
+        req.endpoint = "http://something9999999999999999999999.com"
+        req.service = "SoftLayer_Service"
+        req.method = "getObject"
+        req.mask = "filteredMask[something[nested]]"
+        self.transport(req)
+
+        args, kwargs = request.call_args
+        self.assertIn(
+            "<value><string>filteredMask[something[nested]]</string></value>".encode(),
             kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
@@ -231,14 +249,14 @@ class TestXmlRpcAPICall(testing.TestCase):
         request.return_value = self.response
 
         req = transports.Request()
-        req.endpoint = "http://something.com"
+        req.endpoint = "http://something9999999999999999999999.com"
         req.service = "SoftLayer_Service"
         req.method = "getObject"
         req.mask = "mask.something.nested"
         self.transport(req)
 
         args, kwargs = request.call_args
-        self.assertIn("<value><string>mask.something.nested</string></value>",
+        self.assertIn("<value><string>mask.something.nested</string></value>".encode(),
                       kwargs['data'])
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
@@ -264,6 +282,135 @@ class TestXmlRpcAPICall(testing.TestCase):
         output_text = self.transport.print_reproduceable(req)
         self.assertIn("https://test.com", output_text)
 
+    @mock.patch('SoftLayer.transports.requests.Session.request')
+    @mock.patch('requests.auth.HTTPBasicAuth')
+    def test_ibm_id_call(self, auth, request):
+        request.return_value = self.response
+
+        data = '''<?xml version='1.0' encoding='iso-8859-1'?>
+<methodCall>
+<methodName>getObject</methodName>
+<params>
+<param>
+<value><struct>
+<member>
+<name>headers</name>
+<value><struct>
+</struct></value>
+</member>
+</struct></value>
+</param>
+</params>
+</methodCall>
+'''.encode()
+
+        req = transports.Request()
+        req.service = 'SoftLayer_Service'
+        req.method = 'getObject'
+        req.transport_user = 'apikey'
+        req.transport_password = '1234567890qweasdzxc'
+        resp = self.transport(req)
+
+        auth.assert_called_with('apikey', '1234567890qweasdzxc')
+        request.assert_called_with('POST',
+                                   'http://something9999999999999999999999.com/SoftLayer_Service',
+                                   headers={'Content-Type': 'application/xml',
+                                            'User-Agent': consts.USER_AGENT},
+                                   proxies=None,
+                                   data=data,
+                                   timeout=None,
+                                   cert=None,
+                                   verify=True,
+                                   auth=mock.ANY)
+        self.assertEqual(resp, [])
+        self.assertIsInstance(resp, transports.SoftLayerListResult)
+        self.assertEqual(resp.total_count, 10)
+
+    @mock.patch('SoftLayer.transports.requests.Session.request')
+    def test_call_large_number_response(self, request):
+        response = requests.Response()
+        body = b'''<?xml version="1.0" encoding="utf-8"?>
+<params>
+<param>
+ <value>
+  <array>
+   <data>
+    <value>
+     <struct>
+      <member>
+       <name>bytesUsed</name>
+       <value><int>2666148982056</int></value>
+      </member>
+     </struct>
+    </value>
+   </data>
+  </array>
+ </value>
+</param>
+</params>
+        '''
+        response.raw = io.BytesIO(body)
+        response.headers['SoftLayer-Total-Items'] = 1
+        response.status_code = 200
+        request.return_value = response
+
+        req = transports.Request()
+        req.service = 'SoftLayer_Service'
+        req.method = 'getObject'
+        resp = self.transport(req)
+        self.assertEqual(resp[0]['bytesUsed'], 2666148982056)
+
+    @mock.patch('SoftLayer.transports.requests.Session.request')
+    def test_nonascii_characters(self, request):
+        request.return_value = self.response
+        hostname = 'testé'
+        data = '''<?xml version='1.0' encoding='iso-8859-1'?>
+<methodCall>
+<methodName>getObject</methodName>
+<params>
+<param>
+<value><struct>
+<member>
+<name>headers</name>
+<value><struct>
+</struct></value>
+</member>
+</struct></value>
+</param>
+<param>
+<value><struct>
+<member>
+<name>hostname</name>
+<value><string>testé</string></value>
+</member>
+</struct></value>
+</param>
+</params>
+</methodCall>
+'''.encode()
+
+        req = transports.Request()
+        req.service = 'SoftLayer_Service'
+        req.method = 'getObject'
+        req.args = ({'hostname': hostname},)
+        req.transport_user = "testUser"
+        req.transport_password = "testApiKey"
+        resp = self.transport(req)
+
+        request.assert_called_with('POST',
+                                   'http://something9999999999999999999999.com/SoftLayer_Service',
+                                   headers={'Content-Type': 'application/xml',
+                                            'User-Agent': consts.USER_AGENT},
+                                   proxies=None,
+                                   data=data,
+                                   timeout=None,
+                                   cert=None,
+                                   verify=True,
+                                   auth=mock.ANY)
+        self.assertEqual(resp, [])
+        self.assertIsInstance(resp, transports.SoftLayerListResult)
+        self.assertEqual(resp.total_count, 10)
+
 
 @mock.patch('SoftLayer.transports.requests.Session.request')
 @pytest.mark.parametrize(
@@ -289,7 +436,7 @@ def test_verify(request,
     request.return_value = get_xmlrpc_response()
 
     transport = transports.XmlRpcTransport(
-        endpoint_url='http://something.com',
+        endpoint_url='http://something9999999999999999999999.com',
     )
 
     req = transports.Request()
@@ -305,20 +452,21 @@ def test_verify(request,
     transport(req)
 
     request.assert_called_with('POST',
-                               'http://something.com/SoftLayer_Service',
+                               'http://something9999999999999999999999.com/SoftLayer_Service',
                                data=mock.ANY,
                                headers=mock.ANY,
                                cert=mock.ANY,
                                proxies=mock.ANY,
                                timeout=mock.ANY,
-                               verify=expected)
+                               verify=expected,
+                               auth=None)
 
 
 class TestRestAPICall(testing.TestCase):
 
     def set_up(self):
         self.transport = transports.RestTransport(
-            endpoint_url='http://something.com',
+            endpoint_url='http://something9999999999999999999999.com',
         )
 
     @mock.patch('SoftLayer.transports.requests.Session.request')
@@ -338,7 +486,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertIsInstance(resp, transports.SoftLayerListResult)
         self.assertEqual(resp.total_count, 10)
         request.assert_called_with(
-            'GET', 'http://something.com/SoftLayer_Service/Resource.json',
+            'GET', 'http://something9999999999999999999999.com/SoftLayer_Service/Resource.json',
             headers=mock.ANY,
             auth=None,
             data=None,
@@ -404,7 +552,6 @@ class TestRestAPICall(testing.TestCase):
         req.service = 'SoftLayer_Service'
         req.method = 'Resource'
         req.proxy = 'localhost:3128'
-
         try:
             self.assertRaises(SoftLayer.TransportError, self.transport, req)
         except AssertionError:
@@ -422,7 +569,7 @@ class TestRestAPICall(testing.TestCase):
         self.transport(req)
 
         request.assert_called_with(
-            'GET', 'http://something.com/SoftLayer_Service/Resource.json',
+            'GET', 'http://something9999999999999999999999.com/SoftLayer_Service/Resource.json',
             proxies={'https': 'http://localhost:3128',
                      'http': 'http://localhost:3128'},
             auth=None,
@@ -447,7 +594,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/2/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/2/getObject.json',
             headers=mock.ANY,
             auth=None,
             data=None,
@@ -471,7 +618,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'POST',
-            'http://something.com/SoftLayer_Service/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/getObject.json',
             headers=mock.ANY,
             auth=None,
             data='{"parameters": ["test", 1]}',
@@ -481,6 +628,30 @@ class TestRestAPICall(testing.TestCase):
             proxies=None,
             timeout=None)
 
+    @mock.patch('SoftLayer.transports.requests.Session.request')
+    def test_with_args_bytes(self, request):
+        request().text = '{}'
+
+        req = transports.Request()
+        req.service = 'SoftLayer_Service'
+        req.method = 'getObject'
+        req.args = ('test', b'asdf')
+
+        resp = self.transport(req)
+
+        self.assertEqual(resp, {})
+        request.assert_called_with(
+            'POST',
+            'http://something9999999999999999999999.com/SoftLayer_Service/getObject.json',
+            headers=mock.ANY,
+            auth=None,
+            data='{"parameters": ["test", "YXNkZg=="]}',
+            params={},
+            verify=True,
+            cert=None,
+            proxies=None,
+            timeout=None)
+
     @mock.patch('SoftLayer.transports.requests.Session.request')
     def test_with_filter(self, request):
         request().text = '{}'
@@ -495,7 +666,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/getObject.json',
             params={'objectFilter':
                     '{"TYPE": {"attribute": {"operation": "^= prefix"}}}'},
             headers=mock.ANY,
@@ -520,7 +691,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/getObject.json',
             params={'objectMask': 'mask[id,property]'},
             headers=mock.ANY,
             auth=None,
@@ -541,7 +712,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/getObject.json',
             params={'objectMask': 'mask[id,property]'},
             headers=mock.ANY,
             auth=None,
@@ -567,7 +738,7 @@ class TestRestAPICall(testing.TestCase):
         self.assertEqual(resp, {})
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/2/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/2/getObject.json',
             headers=mock.ANY,
             auth=None,
             data=None,
@@ -610,7 +781,7 @@ class TestRestAPICall(testing.TestCase):
         auth.assert_called_with(user, password)
         request.assert_called_with(
             'GET',
-            'http://something.com/SoftLayer_Service/2/getObject.json',
+            'http://something9999999999999999999999.com/SoftLayer_Service/2/getObject.json',
             headers=mock.ANY,
             auth=mock.ANY,
             data=None,
@@ -628,6 +799,16 @@ class TestRestAPICall(testing.TestCase):
         output_text = self.transport.print_reproduceable(req)
         self.assertIn("https://test.com", output_text)
 
+    def test_complex_encoder_bytes(self):
+        to_encode = {
+            'test': ['array', 0, 1, False],
+            'bytes': b'ASDASDASD'
+        }
+        result = json.dumps(to_encode, cls=transports.ComplexEncoder)
+        # result = '{"test": ["array", 0, 1, false], "bytes": "QVNEQVNEQVNE"}'
+        # encode doesn't always encode in the same order, so testing exact match SOMETIMES breaks.
+        self.assertIn("QVNEQVNEQVNE", result)
+
 
 class TestFixtureTransport(testing.TestCase):
 
